@@ -22,12 +22,13 @@ use Icinga\Module\Director\Web\Widget\ActivityLogInfo;
 use Icinga\Module\Director\Web\Widget\DeployedConfigInfoHeader;
 use Icinga\Module\Director\Web\Widget\ShowConfigFile;
 use Icinga\Web\Notification;
-use Icinga\Web\Url;
 use Exception;
+use RuntimeException;
 use dipl\Html\Html;
 use dipl\Html\HtmlString;
 use dipl\Html\Icon;
 use dipl\Html\Link;
+use dipl\Web\Url;
 
 class ConfigController extends ActionController
 {
@@ -37,6 +38,9 @@ class ConfigController extends ActionController
     {
     }
 
+    /**
+     * @throws \Icinga\Security\SecurityException
+     */
     public function deploymentsAction()
     {
         if ($this->sendNotFoundForRestApi()) {
@@ -77,6 +81,11 @@ class ConfigController extends ActionController
         $table->renderTo($this);
     }
 
+    /**
+     * @throws NotFoundError
+     * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
+     * @throws \Icinga\Security\SecurityException
+     */
     public function deployAction()
     {
         $request = $this->getRequest();
@@ -85,10 +94,10 @@ class ConfigController extends ActionController
         }
 
         if (! $request->isPost()) {
-            throw new IcingaException(
+            throw new RuntimeException(sprintf(
                 'Unsupported method: %s',
                 $request->getMethod()
-            );
+            ));
         }
         $this->assertPermission('director/deploy');
 
@@ -114,6 +123,9 @@ class ConfigController extends ActionController
         }
     }
 
+    /**
+     * @throws \Icinga\Security\SecurityException
+     */
     public function activitiesAction()
     {
         if ($this->sendNotFoundForRestApi()) {
@@ -161,6 +173,11 @@ class ConfigController extends ActionController
         $table->renderTo($this);
     }
 
+    /**
+     * @throws IcingaException
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\ProgrammingError
+     */
     public function activityAction()
     {
         if ($this->sendNotFoundForRestApi()) {
@@ -185,6 +202,9 @@ class ConfigController extends ActionController
         $this->content()->add($info);
     }
 
+    /**
+     * @throws \Icinga\Security\SecurityException
+     */
     public function settingsAction()
     {
         if ($this->sendNotFoundForRestApi()) {
@@ -203,6 +223,9 @@ class ConfigController extends ActionController
 
     /**
      * Show all files for a given config
+     *
+     * @throws \Icinga\Exception\MissingParameterException
+     * @throws \Icinga\Security\SecurityException
      */
     public function filesAction()
     {
@@ -246,6 +269,9 @@ class ConfigController extends ActionController
 
     /**
      * Show a single file
+     *
+     * @throws \Icinga\Exception\MissingParameterException
+     * @throws \Icinga\Security\SecurityException
      */
     public function fileAction()
     {
@@ -276,7 +302,11 @@ class ConfigController extends ActionController
         ));
     }
 
-    // TODO: Check if this can be removed
+    /**
+     * TODO: Check if this can be removed
+     *
+     * @throws \Icinga\Security\SecurityException
+     */
     public function storeAction()
     {
         $this->assertPermission('director/deploy');
@@ -289,6 +319,9 @@ class ConfigController extends ActionController
         );
     }
 
+    /**
+     * @throws \Icinga\Security\SecurityException
+     */
     public function diffAction()
     {
         if ($this->sendNotFoundForRestApi()) {
@@ -310,7 +343,7 @@ class ConfigController extends ActionController
             }
         }
 
-        $this->content()->add(Html::form(['action' => $this->url(), 'method' => 'GET'], [
+        $this->content()->add(Html::tag('form', ['action' => $this->url(), 'method' => 'GET'], [
             new HtmlString($this->view->formSelect(
                 'left',
                 $leftSum,
@@ -336,6 +369,10 @@ class ConfigController extends ActionController
         ConfigFileDiffTable::load($leftSum, $rightSum, $this->db())->renderTo($this);
     }
 
+    /**
+     * @throws IcingaException
+     * @throws \Icinga\Exception\MissingParameterException
+     */
     public function filediffAction()
     {
         if ($this->sendNotFoundForRestApi()) {
@@ -361,6 +398,9 @@ class ConfigController extends ActionController
             ));
     }
 
+    /**
+     * @param $checksum
+     */
     protected function deploymentSucceeded($checksum)
     {
         if ($this->getRequest()->isApiRequest()) {
@@ -375,6 +415,10 @@ class ConfigController extends ActionController
         }
     }
 
+    /**
+     * @param $checksum
+     * @param null $error
+     */
     protected function deploymentFailed($checksum, $error = null)
     {
         $extra = $error ? ': ' . $error: '';
@@ -391,6 +435,9 @@ class ConfigController extends ActionController
         }
     }
 
+    /**
+     * @return \dipl\Web\Widget\Tabs
+     */
     protected function configTabs()
     {
         $tabs = $this->tabs();
