@@ -206,28 +206,11 @@ class ServiceController extends ObjectController
         );
         $table = (new IcingaAppliedServiceTable($this->db()))
             ->setService($service);
-        $table->attributes()->set('data-base-target', '_self');
+        $table->getAttributes()->set('data-base-target', '_self');
 
         $this->content()->add($table);
     }
 
-    public function loadForm($name)
-    {
-        $form = parent::loadForm($name);
-        if ($name === 'icingaService') {
-            if ($this->host) {
-                $form->setHost($this->host);
-            } elseif ($this->set) {
-                $form->setServiceSet($this->set)->setSuccessUrl(
-                    'director/serviceset/services',
-                    array('name' => $this->set->object_name)
-                );
-            }
-        }
-
-        return $form;
-    }
-
     protected function loadObject()
     {
         if ($this->object === null) {
