@@ -6,9 +6,11 @@ use Icinga\Module\Director\Forms\SyncCheckForm;
 use Icinga\Module\Director\Forms\SyncPropertyForm;
 use Icinga\Module\Director\Forms\SyncRuleForm;
 use Icinga\Module\Director\Forms\SyncRunForm;
+use Icinga\Module\Director\Objects\SyncProperty;
 use Icinga\Module\Director\Web\Controller\ActionController;
 use Icinga\Module\Director\Objects\SyncRule;
 use Icinga\Module\Director\Objects\SyncRun;
+use Icinga\Module\Director\Web\Form\CloneSyncRuleForm;
 use Icinga\Module\Director\Web\Table\SyncpropertyTable;
 use Icinga\Module\Director\Web\Table\SyncRunTable;
 use Icinga\Module\Director\Web\Tabs\SyncRuleTabs;
@@ -18,6 +20,12 @@ use dipl\Html\Link;
 
 class SyncruleController extends ActionController
 {
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\IcingaException
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function indexAction()
     {
         $this->setAutoRefreshInterval(10);
@@ -26,6 +34,9 @@ class SyncruleController extends ActionController
         $ruleName = $rule->get('rule_name');
         $this->addTitle($this->translate('Sync rule: %s'), $ruleName);
 
+        $checkForm = SyncCheckForm::load()->setSyncRule($rule)->handleRequest();
+        $runForm = SyncRunForm::load()->setSyncRule($rule)->handleRequest();
+
         if ($lastRunId = $rule->getLastSyncRunId()) {
             $run = SyncRun::load($lastRunId, $this->db());
         } else {
@@ -33,7 +44,7 @@ class SyncruleController extends ActionController
         }
 
         $c = $this->content();
-        $c->add(Html::p($rule->get('description')));
+        $c->add(Html::tag('p', null, $rule->get('description')));
         if (! $rule->hasSyncProperties()) {
             $this->addPropertyHint($rule);
             return;
@@ -45,13 +56,13 @@ class SyncruleController extends ActionController
 
         switch ($rule->get('sync_state')) {
             case 'unknown':
-                $c->add(Html::p($this->translate(
+                $c->add(Html::tag('p', null, $this->translate(
                     "It's currently unknown whether we are in sync with this rule."
                     . ' You should either check for changes or trigger a new Sync Run.'
                 )));
                 break;
             case 'in-sync':
-                $c->add(Html::p(sprintf(
+                $c->add(Html::tag('p', null, sprintf(
                     $this->translate('This Sync Rule was last found to by in Sync at %s.'),
                     $rule->get('last_attempt')
                 )));
@@ -78,14 +89,14 @@ class SyncruleController extends ActionController
                 break;
         }
 
-        $c->add(SyncCheckForm::load()->setSyncRule($rule)->handleRequest());
-        $c->add(SyncRunForm::load()->setSyncRule($rule)->handleRequest());
+        $c->add($checkForm);
+        $c->add($runForm);
 
         if ($run) {
-            $c->add(Html::h3($this->translate('Last sync run details')));
+            $c->add(Html::tag('h3', null, $this->translate('Last sync run details')));
             $c->add(new SyncRunDetails($run));
             if ($run->get('rule_name') !== $ruleName) {
-                $c->add(Html::p(sprintf(
+                $c->add(Html::tag('p', null, sprintf(
                     $this->translate("It has been renamed since then, its former name was %s"),
                     $run->get('rule_name')
                 )));
@@ -93,6 +104,10 @@ class SyncruleController extends ActionController
         }
     }
 
+    /**
+     * @param SyncRule $rule
+     * @throws \Icinga\Exception\IcingaException
+     */
     protected function addPropertyHint(SyncRule $rule)
     {
         $this->warning(Html::sprintf(
@@ -105,21 +120,39 @@ class SyncruleController extends ActionController
         ));
     }
 
+    /**
+     * @param $msg
+     * @throws \Icinga\Exception\IcingaException
+     */
     protected function warning($msg)
     {
-        $this->content()->add(Html::p(['class' => 'warning'], $msg));
+        $this->content()->add(Html::tag('p', ['class' => 'warning'], $msg));
     }
 
+    /**
+     * @param $msg
+     * @throws \Icinga\Exception\IcingaException
+     */
     protected function error($msg)
     {
-        $this->content()->add(Html::p(['class' => 'error'], $msg));
+        $this->content()->add(Html::tag('p', ['class' => 'error'], $msg));
     }
 
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\IcingaException
+     */
     public function addAction()
     {
         $this->editAction();
     }
 
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\IcingaException
+     */
     public function editAction()
     {
         $form = SyncRuleForm::load()
@@ -135,6 +168,14 @@ class SyncruleController extends ActionController
                 $this->translate('Sync rule: %s'),
                 $rule->rule_name
             ));
+            $this->actions()->add(
+                Link::create(
+                    $this->translate('Clone'),
+                    'director/syncrule/clone',
+                    ['id' => $id],
+                    ['class' => 'icon-paste']
+                )
+            );
 
             if (! $rule->hasSyncProperties()) {
                 $this->addPropertyHint($rule);
@@ -148,6 +189,45 @@ class SyncruleController extends ActionController
         $this->content()->add($form);
     }
 
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\MissingParameterException
+     * @throws \Icinga\Exception\NotFoundError
+     * @throws \Icinga\Exception\ProgrammingError
+     */
+    public function cloneAction()
+    {
+        $id = $this->params->getRequired('id');
+        $rule = SyncRule::load($id, $this->db());
+        $this->tabs()->add('show', [
+            'url'       => 'director/syncrule',
+            'urlParams' => ['id' => $id],
+            'label'     => $this->translate('Sync rule'),
+        ])->add('clone', [
+            'url'       => 'director/syncrule/clone',
+            'urlParams' => ['id' => $id],
+            'label'     => $this->translate('Clone'),
+        ])->activate('clone');
+        $this->addTitle('Clone: %s', $rule->get('rule_name'));
+        $this->actions()->add(
+            Link::create(
+                $this->translate('Modify'),
+                'director/syncrule/edit',
+                ['id' => $rule->get('id')],
+                ['class' => 'icon-paste']
+            )
+        );
+
+        $form = new CloneSyncRuleForm($rule);
+        $this->content()->add($form);
+        $form->handleRequest($this->getRequest());
+    }
+
+    /**
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\IcingaException
+     */
     public function propertyAction()
     {
         $rule = $this->requireSyncRule('rule_id');
@@ -166,11 +246,21 @@ class SyncruleController extends ActionController
             ->renderTo($this);
     }
 
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\IcingaException
+     */
     public function editpropertyAction()
     {
         $this->addpropertyAction();
     }
 
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\IcingaException
+     */
     public function addpropertyAction()
     {
         $db = $this->db();
@@ -208,6 +298,12 @@ class SyncruleController extends ActionController
             ->renderTo($this);
     }
 
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\Http\HttpNotFoundException
+     * @throws \Icinga\Exception\IcingaException
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function historyAction()
     {
         $this->setAutoRefreshInterval(30);
@@ -222,6 +318,13 @@ class SyncruleController extends ActionController
         SyncRunTable::create($rule)->renderTo($this);
     }
 
+    /**
+     * @param string $key
+     * @return SyncRule
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\IcingaException
+     * @throws \Icinga\Exception\NotFoundError
+     */
     protected function requireSyncRule($key = 'id')
     {
         $id = $this->params->get($key);
