@@ -2,10 +2,11 @@
 
 namespace Icinga\Module\Director\Forms;
 
-use Icinga\Module\Director\Objects\IcingaHost;
+use Icinga\Exception\AuthenticationException;
 use Icinga\Module\Director\Repository\IcingaTemplateRepository;
+use Icinga\Module\Director\Restriction\HostgroupRestriction;
 use Icinga\Module\Director\Web\Form\DirectorObjectForm;
-use dipl\Html\BaseElement;
+use dipl\Html\BaseHtmlElement;
 use dipl\Html\Html;
 use dipl\Html\Link;
 
@@ -217,11 +218,11 @@ class IcingaHostForm extends DirectorObjectForm
 
         $inherited = $this->getInheritedGroups();
         if (! empty($inherited)) {
-            /** @var BaseElement $links */
+            /** @var BaseHtmlElement $links */
             $links = $this->createHostgroupLinks($inherited);
             if (count($this->object()->getGroups())) {
                 $links->addAttributes(['class' => 'strike-links']);
-                /** @var BaseElement $link */
+                /** @var BaseHtmlElement $link */
                 foreach ($links->getContent() as $link) {
                     $link->addAttributes([
                         'title' => $this->translate(
@@ -241,11 +242,11 @@ class IcingaHostForm extends DirectorObjectForm
         return $this;
     }
 
-    protected function strikeGroupLinks(BaseElement $links)
+    protected function strikeGroupLinks(BaseHtmlElement $links)
     {
-        /** @var BaseElement $link */
+        /** @var BaseHtmlElement $link */
         foreach ($links->getContent() as $link) {
-            $link->attributes()->add('style', 'text-decoration: strike');
+            $link->getAttributes()->add('style', 'text-decoration: strike');
         }
         $links->add('aha');
     }
@@ -253,16 +254,7 @@ class IcingaHostForm extends DirectorObjectForm
     protected function getInheritedGroups()
     {
         if ($this->hasObject()) {
-            $parents = $this->object->imports()->getObjects();
-            /** @var IcingaHost $parent */
-            foreach (array_reverse($parents) as $parent) {
-                $inherited = $parent->getGroups();
-                if (! empty($inherited)) {
-                    return $inherited;
-                }
-            }
-
-            return [];
+            return $this->object->listInheritedGroupNames();
         } else {
             return [];
         }
@@ -375,4 +367,18 @@ class IcingaHostForm extends DirectorObjectForm
 
         return $db->fetchPairs($select);
     }
+
+    public function onSuccess()
+    {
+        if ($this->hasHostGroupRestriction()) {
+            $restriction = new HostgroupRestriction($this->getDb(), $this->getAuth());
+            if (! $restriction->allowsHost($this->object())) {
+                throw new AuthenticationException($this->translate(
+                    'Unable to store a host with the given properties because of insufficient permissions'
+                ));
+            }
+        }
+
+        parent::onSuccess();
+    }
 }
