@@ -3,6 +3,7 @@
 namespace Icinga\Module\Director\Forms;
 
 use Icinga\Data\Filter\Filter;
+use Icinga\Exception\IcingaException;
 use Icinga\Exception\ProgrammingError;
 use Icinga\Module\Director\Data\PropertiesFilter\ArrayCustomVariablesFilter;
 use Icinga\Module\Director\Exception\NestingError;
@@ -10,9 +11,9 @@ use Icinga\Module\Director\Web\Form\DirectorObjectForm;
 use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Objects\IcingaService;
 use Icinga\Module\Director\Objects\IcingaServiceSet;
-use Icinga\Module\Director\Web\Form\Validate\NamePattern;
 use dipl\Html\Html;
 use dipl\Html\Link;
+use RuntimeException;
 
 class IcingaServiceForm extends DirectorObjectForm
 {
@@ -32,35 +33,42 @@ class IcingaServiceForm extends DirectorObjectForm
 
     private $inheritedFrom;
 
+    /** @var bool|null */
+    private $blacklisted;
+
     public function setApplyGenerated(IcingaService $applyGenerated)
     {
         $this->applyGenerated = $applyGenerated;
+
         return $this;
     }
 
     public function setInheritedFrom($hostname)
     {
         $this->inheritedFrom = $hostname;
+
         return $this;
     }
 
+    /**
+     * @throws IcingaException
+     * @throws ProgrammingError
+     * @throws \Zend_Form_Exception
+     */
     public function setup()
     {
+        if (!$this->isNew() || $this->providesOverrides()) {
+            $this->tryToFetchHost();
+        }
+
         if ($this->providesOverrides()) {
             return;
         }
 
         if ($this->host && $this->set) {
             $this->setupOnHostForSet();
-            return;
-        }
 
-        try {
-            if (!$this->isNew() && $this->host === null) {
-                $this->host = $this->object->getResolvedRelated('host');
-            }
-        } catch (NestingError $nestingError) {
-            // ignore for the form to load
+            return;
         }
 
         if ($this->set !== null) {
@@ -72,6 +80,17 @@ class IcingaServiceForm extends DirectorObjectForm
         }
     }
 
+    protected function tryToFetchHost()
+    {
+        try {
+            if ($this->host === null) {
+                $this->host = $this->object->getResolvedRelated('host');
+            }
+        } catch (NestingError $nestingError) {
+            // ignore for the form to load
+        }
+    }
+
     protected function providesOverrides()
     {
         return $this->applyGenerated
@@ -80,45 +99,222 @@ class IcingaServiceForm extends DirectorObjectForm
             || ($this->object && $this->object->usesVarOverrides());
     }
 
+    /**
+     * @throws IcingaException
+     * @throws ProgrammingError
+     * @throws \Zend_Form_Exception
+     */
+    protected function addFields()
+    {
+        if ($this->providesOverrides() && $this->hasBeenBlacklisted()) {
+            $this->onAddedFields();
+
+            return;
+        } else {
+            parent::addFields();
+        }
+    }
+
+    /**
+     * @throws IcingaException
+     * @throws ProgrammingError
+     * @throws \Zend_Form_Exception
+     */
     protected function onAddedFields()
     {
         if (! $this->providesOverrides()) {
             return;
         }
 
-        $this->addOverrideHint();
-
-        $group = $this->getDisplayGroup('custom_fields');
-
-        if ($group) {
-            $elements = $group->getElements();
-            $group->setElements([$this->getElement('inheritance_hint')]);
-            $group->addElements($elements);
-            $this->setSubmitLabel(
-                $this->translate('Override vars')
+        if ($this->hasBeenBlacklisted()) {
+            $this->addHtml(
+                Html::tag(
+                    'p',
+                    ['class' => 'warning'],
+                    $this->translate('This Service has been blacklisted on this host')
+                ),
+                ['name' => 'HINT_blacklisted']
             );
-        } else {
-            $this->addElementsToGroup(
-                array('inheritance_hint'),
-                'custom_fields',
-                20,
-                $this->translate('Hints regarding this service')
-            );
-
+            $group = null;
+            $this->addDeleteButton($this->translate('Restore'));
             $this->setSubmitLabel(false);
+        } else {
+            $this->addOverrideHint();
+            $group = $this->getDisplayGroup('custom_fields');
+            if ($group) {
+                $elements = $group->getElements();
+                $group->setElements([$this->getElement('inheritance_hint')]);
+                $group->addElements($elements);
+                $this->setSubmitLabel($this->translate('Override vars'));
+            } else {
+                $this->addElementsToGroup(
+                    ['inheritance_hint'],
+                    'custom_fields',
+                    20,
+                    $this->translate('Hints regarding this service')
+                );
+
+                $this->setSubmitLabel(false);
+            }
+
+            $this->addDeleteButton($this->translate('Blacklist'));
+        }
+
+        if (! $this->hasSubmitButton()) {
+            $this->addDisplayGroup([$this->deleteButtonName], 'buttons', [
+                'decorators' => [
+                    'FormElements',
+                    ['HtmlTag', ['tag' => 'dl']],
+                    'DtDdWrapper',
+                ],
+                'order' => 1000,
+            ]);
         }
     }
 
+    /**
+     * @param IcingaService $service
+     * @return IcingaService
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    protected function getFirstParent(IcingaService $service)
+    {
+        $objects = $service->imports()->getObjects();
+        if (empty($objects)) {
+            throw new RuntimeException('Something went wrong, got no parent');
+        }
+        reset($objects);
+
+        return current($objects);
+    }
+
+    /**
+     * @return bool
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    protected function hasBeenBlacklisted()
+    {
+        if (! $this->providesOverrides() || $this->object === null) {
+            return false;
+        }
+
+        if ($this->blacklisted === null) {
+            $host = $this->host;
+            $service = $this->getServiceToBeBlacklisted();
+            $db = $this->db->getDbAdapter();
+            if ($this->providesOverrides()) {
+                $this->blacklisted = 1 === (int)$db->fetchOne(
+                    $db->select()->from('icinga_host_service_blacklist', 'COUNT(*)')
+                        ->where('host_id = ?', $host->get('id'))
+                        ->where('service_id = ?', $service->get('id'))
+                );
+            } else {
+                $this->blacklisted = false;
+            }
+        }
+
+        return $this->blacklisted;
+    }
+
+    /**
+     * @param $object
+     * @throws IcingaException
+     * @throws ProgrammingError
+     * @throws \Zend_Db_Adapter_Exception
+     */
+    protected function deleteObject($object)
+    {
+        /** @var IcingaService $object */
+        if ($this->providesOverrides()) {
+            if ($this->hasBeenBlacklisted()) {
+                $this->removeFromBlacklist();
+            } else {
+                $this->blacklist();
+            }
+        } else {
+            parent::deleteObject($object);
+        }
+    }
+
+    /**
+     * @throws IcingaException
+     * @throws \Zend_Db_Adapter_Exception
+     */
+    protected function blacklist()
+    {
+        $host = $this->host;
+        $service = $this->getServiceToBeBlacklisted();
+
+        $db = $this->db->getDbAdapter();
+        $host->unsetOverriddenServiceVars($this->object->getObjectName())->store();
+
+        if ($db->insert('icinga_host_service_blacklist', [
+            'host_id'    => $host->get('id'),
+            'service_id' => $service->get('id')
+        ])) {
+            $msg = sprintf(
+                $this->translate('%s has been blacklisted on %s'),
+                $service->getObjectName(),
+                $host->getObjectName()
+            );
+            $this->redirectOnSuccess($msg);
+        }
+    }
+
+    /**
+     * @return IcingaService
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    protected function getServiceToBeBlacklisted()
+    {
+        if ($this->set) {
+            return $this->object;
+        } else {
+            return $this->getFirstParent($this->object);
+        }
+    }
+
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    protected function removeFromBlacklist()
+    {
+        $host = $this->host;
+        $service = $this->getServiceToBeBlacklisted();
+
+        $db = $this->db->getDbAdapter();
+        $where = implode(' AND ', [
+            $db->quoteInto('host_id = ?', $host->get('id')),
+            $db->quoteInto('service_id = ?', $service->get('id')),
+        ]);
+        if ($db->delete('icinga_host_service_blacklist', $where)) {
+            $msg = sprintf(
+                $this->translate('%s is no longer blacklisted on %s'),
+                $service->getObjectName(),
+                $host->getObjectName()
+            );
+            $this->redirectOnSuccess($msg);
+        }
+    }
+
+    /**
+     * @param IcingaService $service
+     * @return $this
+     */
     public function createApplyRuleFor(IcingaService $service)
     {
         $this->apply = $service;
         $object = $this->object();
         $object->set('imports', $service->getObjectName());
-        $object->object_type = 'apply';
-        $object->object_name = $service->object_name;
+        $object->set('object_type', 'apply');
+        $object->set('object_name', $service->getObjectName());
+
         return $this;
     }
 
+    /**
+     * @throws \Zend_Form_Exception
+     */
     protected function setupServiceElements()
     {
         if ($this->object) {
@@ -145,6 +341,10 @@ class IcingaServiceForm extends DirectorObjectForm
              ->setButtons();
     }
 
+    /**
+     * @throws IcingaException
+     * @throws ProgrammingError
+     */
     protected function addOverrideHint()
     {
         if ($this->object && $this->object->usesVarOverrides()) {
@@ -205,6 +405,10 @@ class IcingaServiceForm extends DirectorObjectForm
         $this->addHtmlHint($hint, ['name' => 'inheritance_hint']);
     }
 
+    /**
+     * @throws IcingaException
+     * @throws ProgrammingError
+     */
     protected function setupOnHostForSet()
     {
         $msg = $this->translate(
@@ -237,18 +441,21 @@ class IcingaServiceForm extends DirectorObjectForm
 
     protected function addAssignmentElements()
     {
-        $this->addAssignFilter(array(
-            'columns' => IcingaHost::enumProperties($this->db, 'host.'),
+        $this->addAssignFilter([
+            'suggestionContext' => 'HostFilterColumns',
             'required' => true,
             'description' => $this->translate(
                 'This allows you to configure an assignment filter. Please feel'
                 . ' free to combine as many nested operators as you want'
             )
-        ));
+        ]);
 
         return $this;
     }
 
+    /**
+     * @throws \Zend_Form_Exception
+     */
     protected function setupHostRelatedElements()
     {
         $this->addHidden('host_id', $this->host->id);
@@ -292,11 +499,15 @@ class IcingaServiceForm extends DirectorObjectForm
         return $this;
     }
 
+    /**
+     * @throws \Zend_Form_Exception
+     */
     protected function setupSetRelatedElements()
     {
         $this->addHidden('service_set_id', $this->set->id);
         $this->addHidden('object_type', 'apply');
         $this->addImportsElement();
+        $this->setButtons();
         $imports = $this->getSentOrObjectValue('imports');
 
         if ($this->hasBeenSent()) {
@@ -310,11 +521,13 @@ class IcingaServiceForm extends DirectorObjectForm
 
         $this->addNameElement()
              ->addDisabledElement()
-             ->groupMainProperties()
-             ->addCheckCommandElements(true)
-             ->addCheckExecutionElements(true)
-             ->addExtraInfoElements()
-             ->setButtons();
+             ->groupMainProperties();
+
+        if ($this->hasPermission('director/admin')) {
+            $this->addCheckCommandElements(true)
+                ->addCheckExecutionElements(true)
+                ->addExtraInfoElements();
+        }
 
         if ($this->hasBeenSent()) {
             $name = $this->getSentOrObjectValue('object_name');
@@ -331,6 +544,10 @@ class IcingaServiceForm extends DirectorObjectForm
         return $this;
     }
 
+    /**
+     * @return $this
+     * @throws \Zend_Form_Exception
+     */
     protected function addNameElement()
     {
         $this->addElement('text', 'object_name', array(
@@ -342,17 +559,16 @@ class IcingaServiceForm extends DirectorObjectForm
         ));
 
         if ($this->object()->isApplyRule()) {
-            $rName = 'director/service/apply/filter-by-name';
-            foreach ($this->getAuth()->getRestrictions($rName) as $restriction) {
-                $this->getElement('object_name')->addValidator(
-                    new NamePattern($restriction)
-                );
-            }
+            $this->eventuallyAddNameRestriction('director/service/apply/filter-by-name');
         }
 
         return $this;
     }
 
+    /**
+     * @return $this
+     * @throws \Zend_Form_Exception
+     */
     protected function addHostObjectElement()
     {
         if ($this->isObject()) {
@@ -369,6 +585,10 @@ class IcingaServiceForm extends DirectorObjectForm
         return $this;
     }
 
+    /**
+     * @return $this
+     * @throws \Zend_Form_Exception
+     */
     protected function addApplyForElement()
     {
         if ($this->object->isApplyRule()) {
@@ -395,6 +615,10 @@ class IcingaServiceForm extends DirectorObjectForm
         return $this;
     }
 
+    /**
+     * @return $this
+     * @throws \Zend_Form_Exception
+     */
     protected function addGroupsElement()
     {
         $groups = $this->enumServicegroups();
@@ -418,6 +642,10 @@ class IcingaServiceForm extends DirectorObjectForm
         return $this;
     }
 
+    /**
+     * @return $this
+     * @throws \Zend_Form_Exception
+     */
     protected function addAgentAndZoneElements()
     {
         if (!$this->isTemplate()) {
@@ -473,6 +701,10 @@ class IcingaServiceForm extends DirectorObjectForm
         return $db->fetchPairs($select);
     }
 
+    /**
+     * @throws IcingaException
+     * @throws ProgrammingError
+     */
     protected function succeedForOverrides()
     {
         $vars = array();
@@ -505,6 +737,10 @@ class IcingaServiceForm extends DirectorObjectForm
         $this->redirectOnSuccess($msg);
     }
 
+    /**
+     * @throws IcingaException
+     * @throws ProgrammingError
+     */
     public function onSuccess()
     {
         if ($this->providesOverrides()) {
