@@ -6,9 +6,15 @@ $this->providePermission(
     'director/showconfig',
     $this->translate('Allow to show configuration (could contain sensitive information)')
 );
+$this->providePermission(
+    'director/showsql',
+    $this->translate('Allow to show the full executed SQL queries in some places')
+);
 $this->providePermission('director/deploy', $this->translate('Allow to deploy configuration'));
 $this->providePermission('director/hosts', $this->translate('Allow to configure hosts'));
 $this->providePermission('director/services', $this->translate('Allow to configure services'));
+$this->providePermission('director/servicesets', $this->translate('Allow to configure service sets'));
+$this->providePermission('director/service_set/apply', $this->translate('Allow to define Service Set Apply Rules'));
 $this->providePermission('director/users', $this->translate('Allow to configure users'));
 $this->providePermission('director/notifications', $this->translate('Allow to configure notifications'));
 $this->providePermission(
@@ -40,6 +46,14 @@ $this->provideRestriction(
     )
 );
 
+$this->provideRestriction(
+    'director/service_set/filter-by-name',
+    $this->translate(
+        'Filter available service set templates. Use asterisks (*) as wildcards,'
+        . ' like in DB* or *net*'
+    )
+);
+
 $this->provideSearchUrl($this->translate('Host configs'), 'director/hosts?limit=10', 60);
 
 /*
