@@ -2,13 +2,15 @@
 
 namespace Icinga\Module\Director\Data\Db;
 
-use Icinga\Exception\IcingaException as IE;
 use Icinga\Exception\NotFoundError;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
 use Icinga\Module\Director\Util;
-use Exception;
+use InvalidArgumentException;
+use LogicException;
+use RuntimeException;
 use Zend_Db_Adapter_Abstract;
+use Zend_Db_Exception;
 
 /**
  * Base class for ...
@@ -24,9 +26,6 @@ abstract class DbObject
     /** @var Zend_Db_Adapter_Abstract */
     protected $db;
 
-    /** @var  Zend_Db_Adapter_Abstract */
-    protected $r;
-
     /**
      * Default columns. MUST be set when extending this class. Each table
      * column MUST be defined with a default value. Default value may be null.
@@ -91,7 +90,7 @@ abstract class DbObject
             || $this->keyName === null
             || $this->defaultProperties === null
         ) {
-            throw new IE("Someone extending this class didn't RTFM");
+            throw new LogicException("Someone extending this class didn't RTFM");
         }
 
         $this->properties = $this->defaultProperties;
@@ -221,6 +220,7 @@ abstract class DbObject
     {
         $this->connection = $connection;
         $this->db = $connection->getDbAdapter();
+
         return $this;
     }
 
@@ -229,8 +229,6 @@ abstract class DbObject
      *
      * @param string $property Property
      *
-     * @throws IE
-     *
      * @return mixed
      */
     public function get($property)
@@ -257,7 +255,10 @@ abstract class DbObject
     protected function assertPropertyExists($key)
     {
         if (! array_key_exists($key, $this->properties)) {
-            throw new IE('Trying to get invalid property "%s"', $key);
+            throw new InvalidArgumentException(sprintf(
+                'Trying to get invalid property "%s"',
+                $key
+            ));
         }
 
         return $this;
@@ -287,8 +288,6 @@ abstract class DbObject
      * @param string $key
      * @param mixed  $value
      *
-     * @throws IE
-     *
      * @return self
      */
     public function set($key, $value)
@@ -300,7 +299,11 @@ abstract class DbObject
 
         $func = 'validate' . ucfirst($key);
         if (method_exists($this, $func) && $this->$func($value) !== true) {
-            throw new IE('Got invalid value "%s" for "%s"', $value, $key);
+            throw new InvalidArgumentException(sprintf(
+                'Got invalid value "%s" for "%s"',
+                $value,
+                $key
+            ));
         }
         $func = 'munge' . ucfirst($key);
         if (method_exists($this, $func)) {
@@ -317,7 +320,10 @@ abstract class DbObject
         }
 
         if (! $this->hasProperty($key)) {
-            throw new IE('Trying to set invalid key %s', $key);
+            throw new InvalidArgumentException(sprintf(
+                'Trying to set invalid key %s',
+                $key
+            ));
         }
 
         if ((is_numeric($value) || is_string($value))
@@ -327,7 +333,7 @@ abstract class DbObject
         }
 
         if ($key === $this->getAutoincKeyName()  && $this->hasBeenLoadedFromDb()) {
-            throw new IE('Changing autoincremental key is not allowed');
+            throw new InvalidArgumentException('Changing autoincremental key is not allowed');
         }
 
         return $this->reallySet($key, $value);
@@ -385,13 +391,12 @@ abstract class DbObject
      * Magic unsetter
      *
      * @param string $key
-     * @throws IE
      * @return void
      */
     public function __unset($key)
     {
         if (! array_key_exists($key, $this->properties)) {
-            throw new IE('Trying to unset invalid key');
+            throw new InvalidArgumentException('Trying to unset invalid key');
         }
         $this->properties[$key] = $this->defaultProperties[$key];
     }
@@ -400,13 +405,15 @@ abstract class DbObject
      * Runs set() for every key/value pair of the given Array
      *
      * @param  array $props  Array of properties
-     * @throws IE
      * @return self
      */
     public function setProperties($props)
     {
         if (! is_array($props)) {
-            throw new IE('Array required, got %s', gettype($props));
+            throw new InvalidArgumentException(sprintf(
+                'Array required, got %s',
+                gettype($props)
+            ));
         }
         foreach ($props as $key => $value) {
             $this->set($key, $value);
@@ -529,7 +536,7 @@ abstract class DbObject
      *
      * // TODO: may conflict with ->id
      *
-     * @return string
+     * @return string|array
      */
     public function getId()
     {
@@ -542,6 +549,7 @@ abstract class DbObject
                 }
                 $id[$key] = $this->properties[$key];
             }
+
             return $id;
         } else {
             if (isset($this->properties[$this->keyName])) {
@@ -640,11 +648,11 @@ abstract class DbObject
     {
         foreach ($properties as $key => $val) {
             if (! array_key_exists($key, $this->properties)) {
-                throw new IE(
+                throw new LogicException(sprintf(
                     'Trying to set invalid %s key "%s". DB schema change?',
                     $this->table,
                     $key
-                );
+                ));
             }
             if ($val === null) {
                 $this->properties[$key] = null;
@@ -697,6 +705,7 @@ abstract class DbObject
      * Ändert den entsprechenden Datensatz in der Datenbank
      *
      * @return int  Anzahl der geänderten Zeilen
+     * @throws \Zend_Db_Adapter_Exception
      */
     protected function updateDb()
     {
@@ -718,6 +727,7 @@ abstract class DbObject
      * Fügt der Datenbank-Tabelle einen entsprechenden Datensatz hinzu
      *
      * @return int  Anzahl der betroffenen Zeilen
+     * @throws \Zend_Db_Adapter_Exception
      */
     protected function insertIntoDb()
     {
@@ -742,7 +752,7 @@ abstract class DbObject
      *
      * @param  DbConnection $db
      * @return bool Whether storing succeeded
-     * @throws IE
+     * @throws DuplicateKeyException
      */
     public function store(DbConnection $db = null)
     {
@@ -751,7 +761,11 @@ abstract class DbObject
         }
 
         if ($this->validate() !== true) {
-            throw new IE('%s[%s] validation failed', $this->table, $this->getLogId());
+            throw new InvalidArgumentException(sprintf(
+                '%s[%s] validation failed',
+                $this->table,
+                $this->getLogId()
+            ));
         }
 
         if ($this->hasBeenLoadedFromDb() && ! $this->hasBeenModified()) {
@@ -769,11 +783,11 @@ abstract class DbObject
                     $result = true;
                     $this->onUpdate();
                 } else {
-                    throw new IE(
+                    throw new RuntimeException(sprintf(
                         'FAILED storing %s "%s"',
                         $table,
                         $this->getLogId()
-                    );
+                    ));
                 }
             } else {
                 if ($id && $this->existsInDb()) {
@@ -799,25 +813,21 @@ abstract class DbObject
                     $this->onInsert();
                     $result = true;
                 } else {
-                    throw new IE(
+                    throw new RuntimeException(sprintf(
                         'FAILED to store new %s "%s"',
                         $table,
                         $this->getLogId()
-                    );
+                    ));
                 }
             }
-        } catch (Exception $e) {
-            if ($e instanceof IE) {
-                throw $e;
-            }
-
-            throw new IE(
+        } catch (Zend_Db_Exception $e) {
+            throw new RuntimeException(sprintf(
                 'Storing %s[%s] failed: %s {%s}',
                 $this->table,
                 $this->getLogId(),
                 $e->getMessage(),
                 var_export($this->getProperties(), 1) // TODO: Remove properties
-            );
+            ));
         }
 
         $this->modifiedProperties = array();
@@ -825,6 +835,7 @@ abstract class DbObject
         $this->loadedProperties = $this->properties;
         $this->onStore();
         $this->loadedFromDb = true;
+
         return $result;
     }
 
@@ -844,17 +855,17 @@ abstract class DbObject
     /**
      * @param string $key
      * @return self
-     * @throws IE
+     * @throws InvalidArgumentException
      */
     protected function setKey($key)
     {
         $keyname = $this->getKeyName();
         if (is_array($keyname)) {
             if (! is_array($key)) {
-                throw new IE(
+                throw new InvalidArgumentException(sprintf(
                     '%s has a multicolumn key, array required',
                     $this->table
-                );
+                ));
             }
             foreach ($keyname as $k) {
                 if (! array_key_exists($k, $key)) {
@@ -945,27 +956,27 @@ abstract class DbObject
         $table = $this->table;
 
         if (! $this->hasBeenLoadedFromDb()) {
-            throw new IE(
+            throw new LogicException(sprintf(
                 'Cannot delete %s "%s", it has not been loaded from Db',
                 $table,
                 $this->getLogId()
-            );
+            ));
         }
 
         if (! $this->existsInDb()) {
-            throw new IE(
+            throw new InvalidArgumentException(sprintf(
                 'Cannot delete %s "%s", it does not exist',
                 $table,
                 $this->getLogId()
-            );
+            ));
         }
         $this->beforeDelete();
         if (! $this->deleteFromDb()) {
-            throw new IE(
+            throw new RuntimeException(sprintf(
                 'Deleting %s (%s) FAILED',
                 $table,
                 $this->getLogId()
-            );
+            ));
         }
         // $this->log(sprintf('%s "%s" has been DELETED', $table, this->getLogId()));
         $this->onDelete();
@@ -1007,6 +1018,10 @@ abstract class DbObject
         return array_key_exists($class, self::$prefetched);
     }
 
+    /**
+     * @param $key
+     * @return static|bool
+     */
     protected static function getPrefetched($key)
     {
         $class = get_called_class();
@@ -1068,6 +1083,12 @@ abstract class DbObject
         return self::$prefetchStats;
     }
 
+    /**
+     * @param $id
+     * @param DbConnection $connection
+     * @return static
+     * @throws NotFoundError
+     */
     public static function loadWithAutoIncId($id, DbConnection $connection)
     {
         /* Need to cast to int, otherwise the id will be matched against
@@ -1087,9 +1108,16 @@ abstract class DbObject
         $obj->setConnection($connection)
             ->set($obj->autoincKeyName, $id)
             ->loadFromDb();
+
         return $obj;
     }
 
+    /**
+     * @param $id
+     * @param DbConnection $connection
+     * @return static
+     * @throws NotFoundError
+     */
     public static function load($id, DbConnection $connection)
     {
         if ($prefetched = static::getPrefetched($id)) {
@@ -1099,6 +1127,7 @@ abstract class DbObject
         /** @var DbObject $obj */
         $obj = new static;
         $obj->setConnection($connection)->setKey($id)->loadFromDb();
+
         return $obj;
     }
 
@@ -1120,7 +1149,6 @@ abstract class DbObject
         } else {
             $select = $query;
         }
-
         $rows = $db->fetchAll($select);
 
         foreach ($rows as $row) {
