@@ -2,10 +2,15 @@
 
 namespace Icinga\Module\Director;
 
+use DateTime;
+use DateTimeZone;
+use Exception;
+use Icinga\Data\ResourceFactory;
+use Icinga\Exception\ConfigurationError;
 use Icinga\Module\Director\Data\Db\DbConnection;
 use Icinga\Module\Director\Objects\IcingaEndpoint;
 use Icinga\Module\Director\Objects\IcingaObject;
-use Icinga\Exception\ConfigurationError;
+use RuntimeException;
 use Zend_Db_Expr;
 use Zend_Db_Select;
 
@@ -26,6 +31,63 @@ class Db extends DbConnection
         return $this->getDbAdapter();
     }
 
+    public function runFailSafeTransaction($callable)
+    {
+        if (! is_callable($callable)) {
+            throw new RuntimeException(__METHOD__ . ' needs a Callable');
+        }
+
+        $db = $this->db();
+        $db->beginTransaction();
+        try {
+            $callable();
+            $db->commit();
+        } catch (Exception $e) {
+            $db->rollback();
+            throw $e;
+        }
+
+        return $this;
+    }
+
+    public static function fromResourceName($name)
+    {
+        $connection = new static(ResourceFactory::getResourceConfig($name));
+
+        if ($connection->isMysql()) {
+            $connection->setClientTimezoneForMysql();
+        } elseif ($connection->isPgsql()) {
+            $connection->setClientTimezoneForPgsql();
+        }
+
+        return $connection;
+    }
+
+    protected function getTimezoneOffset()
+    {
+        $tz = new DateTimeZone(date_default_timezone_get());
+        $offset = $tz->getOffset(new DateTime());
+        $prefix = $offset >= 0 ? '+' : '-';
+        $offset = abs($offset);
+
+        $hours = (int) floor($offset / 3600);
+        $minutes = (int) floor(($offset % 3600) / 60);
+
+        return sprintf('%s%d:%02d', $prefix, $hours, $minutes);
+    }
+
+    protected function setClientTimezoneForMysql()
+    {
+        $db = $this->getDbAdapter();
+        $db->query($db->quoteInto('SET time_zone = ?', $this->getTimezoneOffset()));
+    }
+
+    protected function setClientTimezoneForPgsql()
+    {
+        $db = $this->getDbAdapter();
+        $db->query($db->quoteInto('SET TIME ZONE INTERVAL ? HOUR TO MINUTE', $this->getTimezoneOffset()));
+    }
+
     public function countActivitiesSinceLastDeployedConfig(IcingaObject $object = null)
     {
         $db = $this->db();
@@ -205,6 +267,7 @@ class Db extends DbConnection
     {
         $sql = 'SELECT id, object_type, object_name, action_name,'
              . ' old_properties, new_properties, author, change_time,'
+             . ' UNIX_TIMESTAMP(change_time) AS change_time_ts,'
              . ' %s AS checksum, %s AS parent_checksum'
              . ' FROM director_activity_log WHERE id = %d';
 
@@ -249,6 +312,7 @@ class Db extends DbConnection
 
         $sql = 'SELECT id, object_type, object_name, action_name,'
              . ' old_properties, new_properties, author, change_time,'
+             . ' UNIX_TIMESTAMP(change_time) AS change_time_ts,'
              . ' %s AS checksum, %s AS parent_checksum'
              . ' FROM director_activity_log WHERE checksum = ?';
 
