@@ -69,10 +69,11 @@ class SyncUtils
      *
      * Supports also keys pointing to nested structures like vars.disk.sda
      *
-     * @param  object $row    stdClass object providing property values
-     * @param  string $string Variable/property name
+     * @param  object $row  stdClass object providing property values
+     * @param  string $var  Variable/property name
      *
      * @return mixed
+     * @throws IcingaException
      */
     public static function getSpecificValue($row, $var)
     {
@@ -130,7 +131,7 @@ class SyncUtils
     {
         $res = array();
         foreach ($vars as $p) {
-            if (false === ($pos = strpos($p, '.'))) {
+            if (false === ($pos = strpos($p, '.')) || $pos === strlen($p) - 1) {
                 $res[] = $p;
             } else {
                 $res[] = substr($p, 0, $pos);
