@@ -8,46 +8,50 @@ use Icinga\Module\Director\Data\PropertiesFilter;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
 use Icinga\Module\Director\IcingaConfig\IcingaLegacyConfigHelper as c1;
+use Icinga\Module\Director\Objects\Extension\FlappingSupport;
 
 class IcingaHost extends IcingaObject
 {
+    use FlappingSupport;
+
     protected $table = 'icinga_host';
 
     protected $defaultProperties = array(
-        'id'                    => null,
-        'object_name'           => null,
-        'object_type'           => null,
-        'disabled'              => 'n',
-        'display_name'          => null,
-        'address'               => null,
-        'address6'              => null,
-        'check_command_id'      => null,
-        'max_check_attempts'    => null,
-        'check_period_id'       => null,
-        'check_interval'        => null,
-        'retry_interval'        => null,
-        'check_timeout'         => null,
-        'enable_notifications'  => null,
-        'enable_active_checks'  => null,
-        'enable_passive_checks' => null,
-        'enable_event_handler'  => null,
-        'enable_flapping'       => null,
-        'enable_perfdata'       => null,
-        'event_command_id'      => null,
-        'flapping_threshold'    => null,
-        'volatile'              => null,
-        'zone_id'               => null,
-        'command_endpoint_id'   => null,
-        'notes'                 => null,
-        'notes_url'             => null,
-        'action_url'            => null,
-        'icon_image'            => null,
-        'icon_image_alt'        => null,
-        'has_agent'             => null,
-        'master_should_connect' => null,
-        'accept_config'         => null,
-        'api_key'               => null,
-        'template_choice_id'    => null,
+        'id'                      => null,
+        'object_name'             => null,
+        'object_type'             => null,
+        'disabled'                => 'n',
+        'display_name'            => null,
+        'address'                 => null,
+        'address6'                => null,
+        'check_command_id'        => null,
+        'max_check_attempts'      => null,
+        'check_period_id'         => null,
+        'check_interval'          => null,
+        'retry_interval'          => null,
+        'check_timeout'           => null,
+        'enable_notifications'    => null,
+        'enable_active_checks'    => null,
+        'enable_passive_checks'   => null,
+        'enable_event_handler'    => null,
+        'enable_flapping'         => null,
+        'enable_perfdata'         => null,
+        'event_command_id'        => null,
+        'flapping_threshold_high' => null,
+        'flapping_threshold_low'  => null,
+        'volatile'                => null,
+        'zone_id'                 => null,
+        'command_endpoint_id'     => null,
+        'notes'                   => null,
+        'notes_url'               => null,
+        'action_url'              => null,
+        'icon_image'              => null,
+        'icon_image_alt'          => null,
+        'has_agent'               => null,
+        'master_should_connect'   => null,
+        'accept_config'           => null,
+        'api_key'                 => null,
+        'template_choice_id'      => null,
     );
 
     protected $relations = array(
@@ -102,7 +106,7 @@ class IcingaHost extends IcingaObject
         if ($filter === null) {
             $filter = new PropertiesFilter();
         }
-        $realProperties = static::create()->listProperties();
+        $realProperties = array_merge(['templates'], static::create()->listProperties());
         sort($realProperties);
 
         if ($filter->match(PropertiesFilter::$HOST_PROPERTY, 'name')) {
