@@ -6,7 +6,7 @@ use Exception;
 use Icinga\Data\Filter\Filter;
 use Icinga\Data\Filter\FilterChain;
 use Icinga\Data\Filter\FilterExpression;
-use Icinga\Exception\ProgrammingError;
+use Icinga\Exception\NotFoundError;
 use Icinga\Module\Director\CustomVariable\CustomVariables;
 use Icinga\Module\Director\IcingaConfig\AssignRenderer;
 use Icinga\Module\Director\Data\Db\DbObject;
@@ -19,6 +19,9 @@ use Icinga\Module\Director\IcingaConfig\IcingaConfigRenderer;
 use Icinga\Module\Director\IcingaConfig\IcingaConfigHelper as c;
 use Icinga\Module\Director\IcingaConfig\IcingaLegacyConfigHelper as c1;
 use Icinga\Module\Director\Repository\IcingaTemplateRepository;
+use InvalidArgumentException;
+use LogicException;
+use RuntimeException;
 
 abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 {
@@ -60,22 +63,22 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     protected $type;
 
     /* key/value!! */
-    protected $booleans = array();
+    protected $booleans = [];
 
     // Property suffixed with _id must exist
-    protected $relations = array(
+    protected $relations = [
         // property => PropertyClass
-    );
+    ];
 
-    protected $relatedSets = array(
+    protected $relatedSets = [
         // property => ExtensibleSetClass
-    );
+    ];
 
-    protected $multiRelations = array(
+    protected $multiRelations = [
         // property => IcingaObjectClass
-    );
+    ];
 
-    protected $loadedMultiRelations = array();
+    protected $loadedMultiRelations = [];
 
     /**
      * Allows to set properties pointing to related objects by name without
@@ -83,18 +86,18 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      *
      * @var array
      */
-    protected $unresolvedRelatedProperties = array();
+    protected $unresolvedRelatedProperties = [];
 
-    protected $loadedRelatedSets = array();
+    protected $loadedRelatedSets = [];
 
     // Will be rendered first, before imports
-    protected $prioritizedProperties = array();
+    protected $prioritizedProperties = [];
 
-    protected $propertiesNotForRendering = array(
+    protected $propertiesNotForRendering = [
         'id',
         'object_name',
         'object_type',
-    );
+    ];
 
     /**
      * Array of interval property names
@@ -104,13 +107,14 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      *
      * @var array
      */
-    protected $intervalProperties = array();
+    protected $intervalProperties = [];
 
     /** @var  Db */
     protected $connection;
 
     private $vars;
 
+    /** @var IcingaObjectGroups */
     private $groups;
 
     private $imports;
@@ -120,7 +124,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     private $shouldBeRemoved = false;
 
-    private $resolveCache = array();
+    private $resolveCache = [];
 
     private $cachedPlainUnmodified;
 
@@ -291,23 +295,47 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this->relations[$property];
     }
 
+    /**
+     * @param $property
+     * @return IcingaObject
+     */
     public function getRelated($property)
     {
         return $this->getRelatedObject($property, $this->{$property . '_id'});
     }
 
+    /**
+     * @param $property
+     * @param $id
+     * @return string
+     */
     protected function getRelatedObjectName($property, $id)
     {
-        return $this->getRelatedObject($property, $id)->object_name;
+        return $this->getRelatedObject($property, $id)->getObjectName();
     }
 
+    /**
+     * @param $property
+     * @param $id
+     * @return IcingaObject
+     */
     protected function getRelatedObject($property, $id)
     {
         /** @var IcingaObject $class */
         $class = $this->getRelationClass($property);
-        return $class::loadWithAutoIncId($id, $this->connection);
+        try {
+            $object = $class::loadWithAutoIncId($id, $this->connection);
+        } catch (NotFoundError $e) {
+            throw new RuntimeException($e->getMessage(), 0, $e);
+        }
+
+        return $object;
     }
 
+    /**
+     * @param $property
+     * @return IcingaObject|null
+     */
     public function getResolvedRelated($property)
     {
         $id = $this->getSingleResolvedProperty($property . '_id');
@@ -333,7 +361,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         /** @var static $class */
         $class = self::classByType($type);
         /** @var static $dummy */
-        $dummy = $class::create(array(), $db);
+        $dummy = $class::create([], $db);
         $dummy->prefetchAllRelatedTypes();
     }
 
@@ -447,7 +475,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      *
      * @param Filter|string $filter
      *
-     * @throws ProgrammingError
+     * @throws LogicException
      *
      * @return self
      */
@@ -460,11 +488,11 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                 $type = get_class($this);
             }
 
-            throw new ProgrammingError(
+            throw new LogicException(sprintf(
                 'I can only assign for applied objects or objects with native'
                 . ' support for assigments, got %s',
                 $type
-            );
+            ));
         }
 
         // @codingStandardsIgnoreEnd
@@ -493,20 +521,30 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this;
     }
 
+    /**
+     * @param $name
+     */
     protected function resolveUnresolvedRelatedProperty($name)
     {
         $short = substr($name, 0, -3);
         /** @var IcingaObject $class */
         $class = $this->getRelationClass($short);
-        $object = $class::load(
-            $this->unresolvedRelatedProperties[$name],
-            $this->connection
-        );
+        try {
+            $object = $class::load(
+                $this->unresolvedRelatedProperties[$name],
+                $this->connection
+            );
+        } catch (NotFoundError $e) {
+            throw new RuntimeException($e->getMessage(), 0, $e);
+        }
 
         $this->reallySet($name, $object->get('id'));
         unset($this->unresolvedRelatedProperties[$name]);
     }
 
+    /**
+     * @return bool
+     */
     public function hasBeenModified()
     {
         if (parent::hasBeenModified()) {
@@ -570,6 +608,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return array_key_exists($name, $this->unresolvedRelatedProperties);
     }
 
+    /**
+     * @param $key
+     * @return mixed
+     */
     protected function getRelationId($key)
     {
         if ($this->hasUnresolvedRelatedProperty($key)) {
@@ -579,6 +621,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return parent::get($key);
     }
 
+    /**
+     * @param $key
+     * @return string|null
+     */
     protected function getRelatedProperty($key)
     {
         $idKey = $key . '_id';
@@ -589,13 +635,22 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         if ($id = $this->get($idKey)) {
             /** @var IcingaObject $class */
             $class = $this->getRelationClass($key);
-            $object = $class::loadWithAutoIncId($id, $this->connection);
-            return $object->get('object_name');
+            try {
+                $object = $class::loadWithAutoIncId($id, $this->connection);
+            } catch (NotFoundError $e) {
+                throw new RuntimeException($e->getMessage(), 0, $e);
+            }
+
+            return $object->getObjectName();
         }
 
         return null;
     }
 
+    /**
+     * @param string $key
+     * @return \Icinga\Module\Director\CustomVariable\CustomVariable|mixed|null
+     */
     public function get($key)
     {
         if (substr($key, 0, 5) === 'vars.') {
@@ -738,10 +793,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         } elseif ($value === '' || $value === null) {
             return null;
         } else {
-            throw new ProgrammingError(
+            throw new InvalidArgumentException(sprintf(
                 'Got invalid boolean: %s',
                 var_export($value, 1)
-            );
+            ));
         }
     }
 
@@ -773,7 +828,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     }
 
     /**
-     * @return IcingaObjectGroups[]
+     * @return IcingaObjectGroups
      */
     public function groups()
     {
@@ -789,6 +844,15 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this->groups;
     }
 
+    public function hasModifiedGroups()
+    {
+        $this->assertGroupsSupport();
+        if ($this->groups === null) {
+            return false;
+        }
+
+        return $this->groups->hasBeenModified();
+    }
     /**
      * @return IcingaTimePeriodRanges
      */
@@ -1005,17 +1069,67 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $vars;
     }
 
+    /**
+     * This is mostly for magic getters
+     * @return array
+     */
     public function getGroups()
     {
         return $this->groups()->listGroupNames();
     }
 
+    /**
+     * @return array
+     * @throws NotFoundError
+     */
+    public function listInheritedGroupNames()
+    {
+        $parents = $this->imports()->getObjects();
+        /** @var IcingaObject $parent */
+        foreach (array_reverse($parents) as $parent) {
+            $inherited = $parent->getGroups();
+            if (! empty($inherited)) {
+                return $inherited;
+            }
+        }
+
+        return [];
+    }
+
     public function setGroups($groups)
     {
         $this->groups()->set($groups);
         return $this;
     }
 
+    /**
+     * @return array
+     * @throws NotFoundError
+     */
+    public function listResolvedGroupNames()
+    {
+        $groups = $this->groups()->listGroupNames();
+        if (empty($groups)) {
+            return $this->listInheritedGroupNames();
+        }
+
+        return $groups;
+    }
+
+    /**
+     * @param $group
+     * @return bool
+     * @throws NotFoundError
+     */
+    public function hasGroup($group)
+    {
+        if ($group instanceof static) {
+            $group = $group->getObjectName();
+        }
+
+        return in_array($group, $this->listResolvedGroupNames());
+    }
+
     protected function getResolved($what)
     {
         $func = 'resolve' . $what;
@@ -1090,8 +1204,14 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             }
         }
 
-        /** @var IcingaObject $object */
-        foreach (array_reverse($this->imports()->getObjects()) as $object) {
+        /** @var IcingaObject[] $imports */
+        try {
+            $imports = array_reverse($this->imports()->getObjects());
+        } catch (NotFoundError $e) {
+            throw new RuntimeException($e->getMessage(), 0, $e);
+        }
+
+        foreach ($imports as $object) {
             $v = $object->getSingleResolvedProperty($key);
             if (null !== $v) {
                 return $v;
@@ -1184,10 +1304,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     protected function assertCustomVarsSupport()
     {
         if (! $this->supportsCustomVars()) {
-            throw new ProgrammingError(
+            throw new LogicException(sprintf(
                 'Objects of type "%s" have no custom vars',
                 $this->getType()
-            );
+            ));
         }
 
         return $this;
@@ -1196,10 +1316,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     protected function assertGroupsSupport()
     {
         if (! $this->supportsGroups()) {
-            throw new ProgrammingError(
+            throw new LogicException(sprintf(
                 'Objects of type "%s" have no groups',
                 $this->getType()
-            );
+            ));
         }
 
         return $this;
@@ -1208,10 +1328,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     protected function assertRangesSupport()
     {
         if (! $this->supportsRanges()) {
-            throw new ProgrammingError(
+            throw new LogicException(sprintf(
                 'Objects of type "%s" have no ranges',
                 $this->getType()
-            );
+            ));
         }
 
         return $this;
@@ -1220,10 +1340,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     protected function assertImportsSupport()
     {
         if (! $this->supportsImports()) {
-            throw new ProgrammingError(
+            throw new LogicException(sprintf(
                 'Objects of type "%s" have no imports',
                 $this->getType()
-            );
+            ));
         }
 
         return $this;
@@ -1256,6 +1376,16 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this->vars;
     }
 
+    /**
+     * @return bool
+     */
+    public function hasInitializedVars()
+    {
+        $this->assertCustomVarsSupport();
+
+        return $this->vars !== null;
+    }
+
     public function getVarsTableName()
     {
         return $this->getTableName() . '_var';
@@ -1309,6 +1439,11 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             && $this->get('object_type') === 'apply';
     }
 
+    /**
+     * @throws NotFoundError
+     * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
+     * @throws \Zend_Db_Adapter_Exception
+     */
     protected function storeRelatedObjects()
     {
         $this
@@ -1321,6 +1456,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             ->storeArguments();
     }
 
+    /**
+     * @throws NotFoundError
+     */
     protected function beforeStore()
     {
         $this->resolveUnresolvedRelatedProperties();
@@ -1329,12 +1467,22 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         }
     }
 
+    /**
+     * @throws NotFoundError
+     * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
+     * @throws \Zend_Db_Adapter_Exception
+     */
     public function onInsert()
     {
         DirectorActivityLog::logCreation($this, $this->connection);
         $this->storeRelatedObjects();
     }
 
+    /**
+     * @throws NotFoundError
+     * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
+     * @throws \Zend_Db_Adapter_Exception
+     */
     public function onUpdate()
     {
         DirectorActivityLog::logModification($this, $this->connection);
@@ -1395,7 +1543,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     }
 
     /**
-     * @return self
+     * @return $this
+     * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
      */
     protected function storeArguments()
     {
@@ -1411,7 +1560,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     }
 
     /**
-     * @return self
+     * @return $this
      */
     protected function storeRelatedSets()
     {
@@ -1425,7 +1574,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     }
 
     /**
-     * @return self
+     * @return $this
+     * @throws NotFoundError
+     * @throws \Zend_Db_Adapter_Exception
      */
     protected function storeImports()
     {
@@ -1512,7 +1663,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                 && array_key_exists('enable_active_checks', $this->defaultProperties)
             ) {
                 $passive = clone($this);
-                $passive->enable_active_checks = false;
+                $passive->set('enable_active_checks', false);
 
                 $config->configFile(
                     'director/master/' . $filename,
@@ -1522,7 +1673,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         } elseif ($deploymentMode === 'masterless') {
             // no additional config
         } else {
-            throw new ProgrammingError('Unsupported deployment mode: %s' .$deploymentMode);
+            throw new LogicException(sprintf(
+                'Unsupported deployment mode: %s',
+                $deploymentMode
+            ));
         }
 
         $config->configFile(
@@ -1562,6 +1716,26 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $filename;
     }
 
+    /**
+     * @param $zoneId
+     * @param IcingaConfig|null $config
+     * @return string
+     * @throws NotFoundError
+     */
+    protected function getNameForZoneId($zoneId, IcingaConfig $config = null)
+    {
+        // TODO: this is still ugly.
+        if ($config === null) {
+            return IcingaZone::loadWithAutoIncId(
+                $zoneId,
+                $this->getConnection()
+            )->getObjectName();
+        } else {
+            // Config has a lookup cache, is faster:
+            return $config->getZoneName($zoneId);
+        }
+    }
+
     public function getRenderingZone(IcingaConfig $config = null)
     {
         if ($this->hasUnresolvedRelatedProperty('zone_id')) {
@@ -1569,28 +1743,31 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         }
 
         if ($this->hasProperty('zone_id')) {
-            if (! $this->supportsImports()) {
-                if ($zoneId = $this->get('zone_id')) {
-                    // Config has a lookup cache, is faster:
-                    return $config->getZoneName($zoneId);
-                }
-            }
-
             try {
+                if (! $this->supportsImports()) {
+                    if ($zoneId = $this->get('zone_id')) {
+                        return $this->getNameForZoneId($zoneId, $config);
+                    }
+                }
+
                 if ($zoneId = $this->getSingleResolvedProperty('zone_id')) {
-                    // Config has a lookup cache, is faster:
-                    return $config->getZoneName($zoneId);
+                    return $this->getNameForZoneId($zoneId, $config);
                 }
             } catch (Exception $e) {
                 return self::RESOLVE_ERROR;
             }
         }
 
+        return $this->getDefaultZone($config);
+    }
+
+    protected function getDefaultZone(IcingaConfig $config = null)
+    {
         if ($this->prefersGlobalZone()) {
             return $this->connection->getDefaultGlobalZoneName();
+        } else {
+            return $this->connection->getMasterZoneName();
         }
-
-        return $this->connection->getMasterZoneName();
     }
 
     protected function prefersGlobalZone()
@@ -1612,15 +1789,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         if ($ret !== '') {
             $ret .= "\n";
         }
+
         return $ret;
-
-
-        // TODO: parent_host ORDERed by weigth...
-        if ($this->supportsImports()) {
-            return $this->imports()->toConfigString();
-        } else {
-            return '';
-        }
     }
 
     protected function renderLegacyImports()
@@ -1730,7 +1900,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     {
         return c1::renderKeyValue(
             $legacyKey,
-            c1::renderBoolean($this->$property)
+            c1::renderBoolean($this->get($property))
         );
     }
 
@@ -1884,12 +2054,12 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     protected function renderBooleanProperty($key)
     {
-        return c::renderKeyValue($key, c::renderBoolean($this->$key));
+        return c::renderKeyValue($key, c::renderBoolean($this->get($key)));
     }
 
     protected function renderPropertyAsSeconds($key)
     {
-        return c::renderKeyValue($key, c::renderInterval($this->$key));
+        return c::renderKeyValue($key, c::renderInterval($this->get($key)));
     }
 
     protected function renderSuffix()
@@ -2266,14 +2436,15 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     public function getObjectName()
     {
-        if ($this->hasProperty('object_name')) {
-            return $this->get('object_name');
+        $property = static::getKeyColumnName();
+        if ($this->hasProperty($property)) {
+            return $this->get($property);
         } else {
-            // TODO: replace with an exception once finished
-            throw new ProgrammingError(
-                'Trying to access "object_name" for an instance of "%s"',
+            throw new LogicException(sprintf(
+                'Trying to access "%s" for an instance of "%s"',
+                $property,
                 get_class($this)
-            );
+            ));
         }
     }
 
@@ -2282,7 +2453,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         // allow for icinga_host and host
         $type = lcfirst(preg_replace('/^icinga_/', '', $type));
 
-        if (strpos($type, 'data') === false) {
+        // Hint: Sync/Import are not IcingaObjects, this should be reconsidered:
+        if (strpos($type, 'import') === 0 || strpos($type, 'sync') === 0) {
+            $prefix = '';
+        } elseif (strpos($type, 'data') === false) {
             $prefix = 'Icinga';
         } else {
             $prefix = 'Director';
@@ -2330,6 +2504,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      * @param Db $db
      *
      * @return IcingaObject
+     * @throws NotFoundError
      */
     public static function loadByType($type, $id, Db $db)
     {
@@ -2352,18 +2527,32 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $class::exists($id, $db);
     }
 
-    public static function loadAllByType($type, Db $db, $query = null, $keyColumn = 'object_name')
+    public static function getKeyColumnName()
+    {
+        return 'object_name';
+    }
+
+    public static function loadAllByType($type, Db $db, $query = null, $keyColumn = null)
     {
         /** @var DbObject $class */
         $class = self::classByType($type);
 
+        if ($keyColumn === null) {
+            if (method_exists($class, 'getKeyColumnName')) {
+                $keyColumn = $class::getKeyColumnName();
+            }
+        }
+
         if (is_array($class::create()->getKeyName())) {
             return $class::loadAll($db, $query);
         } else {
-            if (PrefetchCache::shouldBeUsed() && $query === null && $keyColumn === 'object_name') {
+            if (PrefetchCache::shouldBeUsed()
+                && $query === null
+                && $keyColumn === static::getKeyColumnName()
+            ) {
                 $result = array();
                 foreach ($class::prefetchAll($db) as $row) {
-                    $result[$row->object_name] = $row;
+                    $result[$row->$keyColumn] = $row;
                 }
 
                 return $result;
@@ -2377,7 +2566,6 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      * @param $type
      * @param Db $db
      * @return IcingaObject[]
-     * @throws ProgrammingError
      */
     public static function loadAllExternalObjectsByType($type, Db $db)
     {
@@ -2386,10 +2574,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         $dummy = $class::create();
 
         if (is_array($dummy->getKeyName())) {
-            throw new ProgrammingError(
+            throw new LogicException(sprintf(
                 'There is no support for loading external objects of type "%s"',
                 $type
-            );
+            ));
         } else {
             $query = $db->getDbAdapter()
                 ->select()
@@ -2410,6 +2598,12 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return static::create((array) $plain, $connection);
     }
 
+    /**
+     * @param IcingaObject $object
+     * @param null $preserve
+     * @return $this
+     * @throws NotFoundError
+     */
     public function replaceWith(IcingaObject $object, $preserve = null)
     {
         if ($preserve === null) {
@@ -2428,7 +2622,14 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this;
     }
 
-    // TODO: with rules? What if I want to override vars? Drop in favour of vars.x?
+    /**
+     * TODO: with rules? What if I want to override vars? Drop in favour of vars.x?
+     *
+     * @param IcingaObject $object
+     * @param bool $replaceVars
+     * @return $this
+     * @throws NotFoundError
+     */
     public function merge(IcingaObject $object, $replaceVars = false)
     {
         $object = clone($object);
@@ -2488,6 +2689,14 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this;
     }
 
+    /**
+     * @param bool $resolved
+     * @param bool $skipDefaults
+     * @param array|null $chosenProperties
+     * @param bool $resolveIds
+     * @return object
+     * @throws NotFoundError
+     */
     public function toPlainObject(
         $resolved = false,
         $skipDefaults = false,
@@ -2527,7 +2736,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
                         $k = $relKey;
                     } else {
-                        throw new ProgrammingError('No such relation: %s', $relKey);
+                        throw new LogicException(sprintf(
+                            'No such relation: %s',
+                            $relKey
+                        ));
                     }
                 }
             }
@@ -2550,7 +2762,12 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
         if ($this->supportsGroups()) {
             // TODO: resolve
-            $props['groups'] = $this->groups()->listGroupNames();
+            $groups = $this->groups()->listGroupNames();
+            if ($resolved && empty($groups)) {
+                $groups = $this->listInheritedGroupNames();
+            }
+
+            $props['groups'] = $groups;
         }
 
         foreach ($this->loadAllMultiRelations() as $key => $rel) {
@@ -2716,6 +2933,13 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return 'director/' . $plural;
     }
 
+    /**
+     * @param bool $resolved
+     * @param bool $skipDefaults
+     * @param array|null $chosenProperties
+     * @return string
+     * @throws NotFoundError
+     */
     public function toJson(
         $resolved = false,
         $skipDefaults = false,
