@@ -3,10 +3,9 @@
 namespace Icinga\Module\Director\Objects;
 
 use Icinga\Data\Filter\Filter;
-use Icinga\Exception\IcingaException;
-use Icinga\Exception\ProgrammingError;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
+use InvalidArgumentException;
 
 class IcingaServiceSet extends IcingaObject
 {
@@ -55,7 +54,10 @@ class IcingaServiceSet extends IcingaObject
                 $this->set('object_name', $keyComponents[0]);
                 $this->set('object_type', 'template');
             } else {
-                throw new IcingaException('Can not parse key: %s', $key);
+                throw new InvalidArgumentException(sprintf(
+                    'Can not parse key: %s',
+                    $key
+                ));
             }
         } else {
             return parent::setKey($key);
@@ -107,6 +109,30 @@ class IcingaServiceSet extends IcingaObject
         return $services;
     }
 
+    public function onDelete()
+    {
+        $hostId = $this->get('host_id');
+        if ($hostId) {
+            $deleteIds = [];
+            foreach ($this->getServiceObjects() as $service) {
+                $deleteIds[] = (int) $service->get('id');
+            }
+
+            if (! empty($deleteIds)) {
+                $db = $this->getDb();
+                $db->delete(
+                    'icinga_host_service_blacklist',
+                    $db->quoteInto(
+                        sprintf('host_id = %s AND service_id IN (?)', $hostId),
+                        $deleteIds
+                    )
+                );
+            }
+        }
+
+        parent::onDelete();
+    }
+
     public function renderToConfig(IcingaConfig $config)
     {
         if ($this->get('assign_filter') === null && $this->isTemplate()) {
@@ -152,7 +178,7 @@ class IcingaServiceSet extends IcingaObject
             'zones.d/' . $this->getRenderingZone($config) . '/servicesets'
         );
 
-        $file->prepend($this->getConfigHeaderComment($config));
+        $file->addContent($this->getConfigHeaderComment($config));
         return $file;
     }
 
@@ -287,7 +313,7 @@ class IcingaServiceSet extends IcingaObject
         $name = $this->getObjectName();
 
         if ($this->isObject() && $this->get('host_id') === null) {
-            throw new ProgrammingError(
+            throw new InvalidArgumentException(
                 'A Service Set cannot be an object with no related host'
             );
         }
