@@ -4,18 +4,15 @@ namespace Icinga\Module\Director\Web\Controller;
 
 use Icinga\Application\Benchmark;
 use Icinga\Data\Paginatable;
+use Icinga\Exception\ProgrammingError;
 use Icinga\Module\Director\Monitoring;
 use Icinga\Module\Director\Web\Controller\Extension\CoreApi;
 use Icinga\Module\Director\Web\Controller\Extension\DirectorDb;
 use Icinga\Module\Director\Web\Controller\Extension\RestApi;
-use Icinga\Module\Director\Web\Form\FormLoader;
-use Icinga\Module\Director\Web\Form\QuickForm;
-use Icinga\Module\Director\Web\Table\QuickTable;
-use Icinga\Module\Director\Web\Table\TableLoader;
 use Icinga\Security\SecurityException;
 use Icinga\Web\Controller;
 use Icinga\Web\UrlParams;
-use Icinga\Web\Widget;
+use InvalidArgumentException;
 use dipl\Compat\Translator;
 use dipl\Html\Link;
 use dipl\Translation\TranslationHelper;
@@ -38,12 +35,18 @@ abstract class ActionController extends Controller implements ControlsAndContent
     /** @var Monitoring */
     private $monitoring;
 
+    /**
+     * @throws SecurityException
+     * @throws \Icinga\Exception\AuthenticationException
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function init()
     {
         $this->initializeTranslator();
         Benchmark::measure('Director base Controller init()');
         $this->checkForRestApiRequest();
         $this->checkDirectorPermissions();
+        $this->checkSpecialDirectorPermissions();
     }
 
     protected function initializeTranslator()
@@ -56,11 +59,24 @@ abstract class ActionController extends Controller implements ControlsAndContent
         return $this->Auth();
     }
 
+    /**
+     * @throws SecurityException
+     */
     protected function checkDirectorPermissions()
     {
         $this->assertPermission('director/admin');
     }
 
+    /**
+     * @throws SecurityException
+     */
+    protected function checkSpecialDirectorPermissions()
+    {
+        if ($this->params->get('format') === 'sql') {
+            $this->assertPermission('director/showsql');
+        }
+    }
+
     /**
      * Assert that the current user has one of the given permission
      *
@@ -92,7 +108,11 @@ abstract class ActionController extends Controller implements ControlsAndContent
     public function setAutorefreshInterval($interval)
     {
         if (! $this->getRequest()->isApiRequest()) {
-            parent::setAutorefreshInterval($interval);
+            try {
+                parent::setAutorefreshInterval($interval);
+            } catch (ProgrammingError $e) {
+                throw new InvalidArgumentException($e->getMessage());
+            }
         }
 
         return $this;
@@ -136,35 +156,10 @@ abstract class ActionController extends Controller implements ControlsAndContent
         return $this;
     }
 
-    /**
-     * @param string $name
-     *
-     * @return QuickForm
-     */
-    public function loadForm($name)
-    {
-        $form = FormLoader::load($name, $this->Module());
-        if ($this->getRequest()->isApiRequest()) {
-            // TODO: Ask form for API support?
-            $form->setApiRequest();
-        }
-
-        return $form;
-    }
-
-    /**
-     * @param string $name
-     *
-     * @return QuickTable
-     */
-    public function loadTable($name)
-    {
-        return TableLoader::load($name, $this->Module());
-    }
-
     /**
      * @param string $permission
      * @return $this
+     * @throws SecurityException
      */
     public function assertPermission($permission)
     {
@@ -181,7 +176,7 @@ abstract class ActionController extends Controller implements ControlsAndContent
             $this->view = $viewRenderer->view;
             if ($this->getOriginalUrl()->getParam('view') === 'compact') {
                 if ($this->view->controls) {
-                    $this->controls()->attributes()->add('style', 'display: none;');
+                    $this->controls()->getAttributes()->add('style', 'display: none;');
                 }
             }
         } else {
