@@ -11,8 +11,10 @@ use Icinga\Module\Director\Exception\NestingError;
 use Icinga\Module\Director\IcingaConfig\StateFilterSet;
 use Icinga\Module\Director\IcingaConfig\TypeFilterSet;
 use Icinga\Module\Director\Objects\IcingaTemplateChoice;
+use Icinga\Module\Director\Objects\IcingaCommand;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Util;
+use Icinga\Module\Director\Web\Form\Validate\NamePattern;
 use Zend_Form_Element as ZfElement;
 use Zend_Form_Element_Select as ZfSelect;
 
@@ -80,6 +82,18 @@ abstract class DirectorObjectForm extends DirectorForm
         return $this->auth;
     }
 
+    protected function eventuallyAddNameRestriction($restrictionName)
+    {
+        $restrictions = $this->getAuth()->getRestrictions($restrictionName);
+        if (! empty($restrictions)) {
+            $this->getElement('object_name')->addValidator(
+                new NamePattern($restrictions)
+            );
+        }
+
+        return $this;
+    }
+
     public function presetImports($imports)
     {
         if (! empty($imports)) {
@@ -767,14 +781,15 @@ abstract class DirectorObjectForm extends DirectorForm
         } catch (Exception $e) {
             $this->addUniqueException($e);
         }
+
+        if ($this->shouldBeDeleted()) {
+            $this->deleteObject($this->object());
+        }
     }
 
     protected function handlePost()
     {
         $object = $this->object();
-        if ($this->shouldBeDeleted()) {
-            $this->deleteObject($object);
-        }
 
         $post = $this->getRequest()->getPost();
         $this->populate($post);
@@ -872,6 +887,15 @@ abstract class DirectorObjectForm extends DirectorForm
                     )
                 );
             }
+        } elseif ($object instanceof IcingaCommand && $object->isInUse()) {
+            $el->setAttrib('disabled', 'disabled');
+            $el->setAttrib(
+                'title',
+                sprintf(
+                    $this->translate('This Command is still in use by %d other objects'),
+                    $object->countDirectUses()
+                )
+            );
         }
 
         $this->addElement($el);
@@ -1092,6 +1116,10 @@ abstract class DirectorObjectForm extends DirectorForm
      */
     protected function addChoices($type)
     {
+        if ($this->isTemplate()) {
+            return $this;
+        }
+
         $connection = $this->getDb();
         $choiceType = 'TemplateChoice' . ucfirst($type);
         $choices = IcingaObject::loadAllByType($choiceType, $connection);
@@ -1114,6 +1142,7 @@ abstract class DirectorObjectForm extends DirectorForm
     /**
      * @param bool $required
      * @return $this
+     * @throws \Zend_Form_Exception
      */
     protected function addImportsElement($required = null)
     {
@@ -1195,6 +1224,10 @@ abstract class DirectorObjectForm extends DirectorForm
         return $this;
     }
 
+    /**
+     * @return $this
+     * @throws \Zend_Form_Exception
+     */
     protected function addGroupDisplayNameElement()
     {
         $this->addElement('text', 'display_name', array(
@@ -1212,6 +1245,7 @@ abstract class DirectorObjectForm extends DirectorForm
      * @param bool $force
      *
      * @return $this
+     * @throws \Zend_Form_Exception
      */
     protected function addCheckCommandElements($force = false)
     {
@@ -1341,6 +1375,34 @@ abstract class DirectorObjectForm extends DirectorForm
             $this->translate('Whether to process performance data provided by this object')
         );
 
+        $this->optionalBoolean(
+            'enable_flapping',
+            $this->translate('Enable flap detection'),
+            $this->translate('Whether flap detection is enabled on this object')
+        );
+
+        $this->addElement(
+            'text',
+            'flapping_threshold_high',
+            array(
+                'label' => $this->translate('Flapping threshold (high)'),
+                'description' => $this->translate(
+                    'Flapping upper bound in percent for a service to be considered flapping'
+                )
+            )
+        );
+
+        $this->addElement(
+            'text',
+            'flapping_threshold_low',
+            array(
+                'label' => $this->translate('Flapping threshold (low)'),
+                'description' => $this->translate(
+                    'Flapping lower bound in percent for a service to be considered not flapping'
+                )
+            )
+        );
+
         $this->optionalBoolean(
             'volatile',
             $this->translate('Volatile'),
@@ -1358,6 +1420,9 @@ abstract class DirectorObjectForm extends DirectorForm
             'enable_notifications',
             'enable_event_handler',
             'enable_perfdata',
+            'enable_flapping',
+            'flapping_threshold_high',
+            'flapping_threshold_low',
             'volatile'
         );
         $this->addToCheckExecutionDisplayGroup($elements);
@@ -1452,9 +1517,10 @@ abstract class DirectorObjectForm extends DirectorForm
      * Forms should use this helper method for objects using the typical
      * assign_filter column
      *
-     * @param array  $properties Form element properties
+     * @param array $properties Form element properties
      *
      * @return $this
+     * @throws \Zend_Form_Exception
      */
     protected function addAssignFilter($properties)
     {
@@ -1484,10 +1550,11 @@ abstract class DirectorObjectForm extends DirectorForm
      * TODO: Evaluate whether parts or all of this could be moved to the element
      * class.
      *
-     * @param string $name       Element name
-     * @param array  $properties Form element properties
+     * @param string $name Element name
+     * @param array $properties Form element properties
      *
      * @return $this
+     * @throws \Zend_Form_Exception
      */
     protected function addFilterElement($name, $properties)
     {
