@@ -121,7 +121,9 @@ class ObjectTabs extends Tabs
             ]);
         }
 
-        if ($object->getShortTableName() === 'endpoint') {
+        if ($object->getShortTableName() === 'endpoint'
+            && $object->get('apiuser_id')
+        ) {
             $this->add('inspect', [
                 'url'       => 'director/inspect/types',
                 'urlParams' => ['endpoint' => $object->getObjectName()],
