@@ -258,7 +258,8 @@
             } else {
                 $ul = $('<ul class="director-suggestions"></ul>');
                 $ul.insertAfter($input);
-                $ul.css({width: $input.css('width')});
+                var suggestionWidth = (parseInt($input.css('width')) * 2) + 'px';
+                $ul.css({width: suggestionWidth});
                 return this.refreshSuggestionList($ul, $input);
             }
         },
@@ -308,6 +309,14 @@
             var $el = $suggestion.closest('ul').siblings('.director-suggest');
             var val = $suggestion.text();
 
+            // extract label and key from key
+            var re = /^(.+) \[(\w+)]$/;
+
+            var withLabel = val.match(re);
+            if (withLabel) {
+                val = withLabel[2];
+            }
+
             if (val.match(/\.$/)) {
                 $el.val(val);
                 this.getSuggestionList($el, true);
@@ -676,6 +685,9 @@
             if (! this.containerIsAutorefreshed($container) && ! this.containerIsAutoSubmitted($container)) {
                 this.putFocusOnFirstFormElement($container);
             }
+
+            // Turn off autocomplete for all suggested fields
+            $container.find('input.director-suggest').each(this.disableAutocomplete);
         },
 
         highlightActiveDashlet: function($container)
@@ -788,6 +800,14 @@
             } else {
                 $fieldset.find('legend span.element-count').remove();
             }
+        },
+
+        disableAutocomplete: function() {
+            $(this)
+                .attr('autocomplete', 'off')
+                .attr('autocorrect', 'off')
+                .attr('autocapitalize', 'off')
+                .attr('spellcheck', 'false');
         }
     };
 
