@@ -455,7 +455,8 @@ CREATE TABLE icinga_host (
   enable_flapping ENUM('y', 'n') DEFAULT NULL,
   enable_perfdata ENUM('y', 'n') DEFAULT NULL,
   event_command_id INT(10) UNSIGNED DEFAULT NULL,
-  flapping_threshold SMALLINT UNSIGNED default null,
+  flapping_threshold_high SMALLINT UNSIGNED default null,
+  flapping_threshold_low SMALLINT UNSIGNED default null,
   volatile ENUM('y', 'n') DEFAULT NULL,
   zone_id INT(10) UNSIGNED DEFAULT NULL,
   command_endpoint_id INT(10) UNSIGNED DEFAULT NULL,
@@ -613,7 +614,8 @@ CREATE TABLE icinga_service (
   enable_flapping ENUM('y', 'n') DEFAULT NULL,
   enable_perfdata ENUM('y', 'n') DEFAULT NULL,
   event_command_id INT(10) UNSIGNED DEFAULT NULL,
-  flapping_threshold SMALLINT UNSIGNED DEFAULT NULL,
+  flapping_threshold_high SMALLINT UNSIGNED DEFAULT NULL,
+  flapping_threshold_low SMALLINT UNSIGNED DEFAULT NULL,
   volatile ENUM('y', 'n') DEFAULT NULL,
   zone_id INT(10) UNSIGNED DEFAULT NULL,
   command_endpoint_id INT(10) UNSIGNED DEFAULT NULL,
@@ -746,6 +748,22 @@ CREATE TABLE icinga_host_service (
     ON UPDATE CASCADE
 ) ENGINE=InnoDB;
 
+CREATE TABLE icinga_host_service_blacklist (
+  host_id INT(10) UNSIGNED NOT NULL,
+  service_id INT(10) UNSIGNED NOT NULL,
+  PRIMARY KEY (host_id, service_id),
+  CONSTRAINT icinga_host_service_bl_host
+  FOREIGN KEY host (host_id)
+  REFERENCES icinga_host (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_host_service_bl_service
+  FOREIGN KEY service (service_id)
+  REFERENCES icinga_service (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+) ENGINE=InnoDB;
+
 CREATE TABLE icinga_service_set_inheritance (
   service_set_id INT(10) UNSIGNED NOT NULL,
   parent_service_set_id INT(10) UNSIGNED NOT NULL,
@@ -1026,9 +1044,15 @@ CREATE TABLE icinga_usergroup (
   object_type ENUM('object', 'template') NOT NULL,
   disabled ENUM('y', 'n') NOT NULL DEFAULT 'n',
   display_name VARCHAR(255) DEFAULT NULL,
+  zone_id INT(10) UNSIGNED DEFAULT NULL,
   PRIMARY KEY (id),
   UNIQUE INDEX object_name (object_name),
-  KEY search_idx (display_name)
+  KEY search_idx (display_name),
+  CONSTRAINT icinga_usergroup_zone
+    FOREIGN KEY zone (zone_id)
+    REFERENCES icinga_zone (id)
+      ON DELETE RESTRICT
+      ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
 CREATE TABLE icinga_usergroup_inheritance (
@@ -1255,7 +1279,7 @@ CREATE TABLE import_source (
   id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
   source_name VARCHAR(64) NOT NULL,
   key_column VARCHAR(64) NOT NULL,
-  provider_class VARCHAR(72) NOT NULL,
+  provider_class VARCHAR(128) NOT NULL,
   import_state ENUM(
     'unknown',
     'in-sync',
@@ -1286,7 +1310,7 @@ CREATE TABLE import_row_modifier (
   source_id INT(10) UNSIGNED NOT NULL,
   property_name VARCHAR(255) NOT NULL,
   target_property VARCHAR(255) DEFAULT NULL,
-  provider_class VARCHAR(72) NOT NULL,
+  provider_class VARCHAR(128) NOT NULL,
   priority SMALLINT UNSIGNED NOT NULL,
   description TEXT DEFAULT NULL,
   PRIMARY KEY (id),
@@ -1668,4 +1692,4 @@ CREATE TABLE icinga_dependency_states_set (
 
 INSERT INTO director_schema_migration
   (schema_version, migration_time)
-  VALUES (145, NOW());
+  VALUES (149, NOW());
