@@ -580,7 +580,8 @@ CREATE TABLE icinga_host (
   enable_flapping enum_boolean DEFAULT NULL,
   enable_perfdata enum_boolean DEFAULT NULL,
   event_command_id integer DEFAULT NULL,
-  flapping_threshold smallint default null,
+  flapping_threshold_high smallint default null,
+  flapping_threshold_low smallint default null,
   volatile enum_boolean DEFAULT NULL,
   zone_id integer DEFAULT NULL,
   command_endpoint_id integer DEFAULT NULL,
@@ -766,7 +767,8 @@ CREATE TABLE icinga_service (
   enable_flapping enum_boolean DEFAULT NULL,
   enable_perfdata enum_boolean DEFAULT NULL,
   event_command_id integer DEFAULT NULL,
-  flapping_threshold smallint DEFAULT NULL,
+  flapping_threshold_high smallint DEFAULT NULL,
+  flapping_threshold_low smallint DEFAULT NULL,
   volatile enum_boolean DEFAULT NULL,
   zone_id integer DEFAULT NULL,
   command_endpoint_id integer DEFAULT NULL,
@@ -925,6 +927,26 @@ CREATE INDEX host_service_host ON icinga_host_service (host_id);
 CREATE INDEX host_service_service ON icinga_host_service (service_id);
 
 
+CREATE TABLE icinga_host_service_blacklist(
+  host_id integer NOT NULL,
+  service_id integer NOT NULL,
+  PRIMARY KEY (host_id, service_id),
+  CONSTRAINT icinga_host_service__bl_host
+  FOREIGN KEY (host_id)
+  REFERENCES icinga_host (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_host_service_bl_service
+  FOREIGN KEY (service_id)
+  REFERENCES icinga_service (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+);
+
+CREATE INDEX host_service_bl_host ON icinga_host_service_blacklist (host_id);
+CREATE INDEX host_service_bl_service ON icinga_host_service_blacklist (service_id);
+
+
 CREATE TABLE icinga_service_set_inheritance (
   service_set_id integer NOT NULL,
   parent_service_set_id integer NOT NULL,
@@ -1245,11 +1267,18 @@ CREATE TABLE icinga_usergroup (
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
   display_name character varying(255) DEFAULT NULL,
-  PRIMARY KEY (id)
+  zone_id integer DEFAULT NULL,
+  PRIMARY KEY (id),
+  CONSTRAINT icinga_usergroup_zone
+    FOREIGN KEY (zone_id)
+    REFERENCES icinga_zone (id)
+      ON DELETE RESTRICT
+      ON UPDATE CASCADE
 );
 
 CREATE UNIQUE INDEX usergroup_search_idx ON icinga_usergroup (display_name);
 CREATE INDEX usergroup_object_name ON icinga_usergroup (object_name);
+CREATE INDEX usergroup_zone ON icinga_usergroup (zone_id);
 
 
 CREATE TABLE icinga_usergroup_inheritance (
@@ -1395,7 +1424,7 @@ CREATE TABLE import_source (
   id serial,
   source_name character varying(64) NOT NULL,
   key_column character varying(64) NOT NULL,
-  provider_class character varying(72) NOT NULL,
+  provider_class character varying(128) NOT NULL,
   import_state enum_sync_state NOT NULL DEFAULT 'unknown',
   last_error_message text NULL DEFAULT NULL,
   last_attempt timestamp with time zone NULL DEFAULT NULL,
@@ -1426,7 +1455,7 @@ CREATE TABLE import_row_modifier (
   source_id integer NOT NULL,
   property_name character varying(255) NOT NULL,
   target_property character varying(255) DEFAULT NULL,
-  provider_class character varying(72) NOT NULL,
+  provider_class character varying(128) NOT NULL,
   priority integer NOT NULL,
   description text DEFAULT NULL,
   PRIMARY KEY (id),
@@ -1959,4 +1988,4 @@ COMMENT ON COLUMN icinga_dependency_states_set.merge_behaviour IS 'override: = [
 
 INSERT INTO director_schema_migration
   (schema_version, migration_time)
-  VALUES (144, NOW());
+  VALUES (149, NOW());
